% a_rawread.m
% This script reads the raw file whose name is passed in fname variable and
% shapes the data in a reasonable and useful format. This means that it
% adds load, generator, switched shunt, and UPFC data to the BUS vectors
% and line and transformer data to BRANCH vectors. All data is extracted
% from RAW file and put in two vectors series bu___ and br___.
%
% Needs: fileName: the raw data file name.
% Defines:  bu___: for bus variables including original bus section, as
%                  well as load and generator data.
%                  buType, buVoltage, buAngle
%                  buPd, buQd, buPg, buQg, buQmin, buQmax.
%           br___: for branch variables. Includes line and transformer
%                  variables.
%                  brFrom, brTo, brR, brX, brB, brRatio, brStat.
%            n___: for number of different element of the system.
%                  nBus, nLoad, nGenerator, nLine, nTransformer, nSwitched,
%                  nFACTS, and later, nBranch.
%         baseMVA: base MVA used for the system.
%
% Part of the code written for GPFM strategy,
% submitted to IEEE Transactions on Smart Grid, July 2011.
%
% Copyright (c) 2012 by Ali Mehrizi-Sani
% Washington State University
% mehrizi@eecs.wsu.edu
% http://eecs.wsu.edu/~mehrizi
% (509) 335-6249


% Open the file
fid = fopen(fileName);

% Read base MVA of the system
tline = fgets(fid);
[IC baseMVA] = strread(tline, '%f %f', 1, 'delimiter', ',');

% Read file header information
tline = fgets(fid);
disp(sprintf('\nUsing case %s', tline(1:end-2)))
tline = fgets(fid);
disp(sprintf('By %s', tline(1:end-2)))

% Start reading data. Note that not all data provided is used. Only the
% data required for a normal and basic Power Flow are recorded.

% 1. Read bus data
tline = fgets(fid);
i = 1;
while (strread(tline, '%d', 1) ~= 0)
    [no buName buBasekv buType(i) GL BL area zone buVoltage(i) buAngle(i) owner] = ... 
        strread(tline, '%d %s %f %d %f %f %d %d %f %f %d', 1, 'delimiter', ',');
    
    tline = fgets(fid);
    i = i+1;
end
nBus = i-1;

% 2. Read load data
tline = fgets(fid);
i = 1;
while strread(tline, '%d', 1) ~= 0
    [loBus(i) loName loStat(i) loArea zone loPL(i) loQL(i) IP IQ YP YQ owner] = ...
        strread(tline, '%d %s %d %d %d %f %f %f %f %f %f %d', 1, 'delimiter', ',');
    
    tline = fgets(fid);
    i = i+1;
end
nLoad = i-1;

% 3. Read gen data
tline = fgets(fid);
i = 1;
while strread(tline, '%d', 1) ~= 0
    [geBus(i) geName gePG(i) geQG(i) geQmax(i) geQmin(i) geVS geIREG geMBase(i) ...
            geZR geZX geRT geXT geGTap geStat(i) geRMPCT gePT gePB geO geF] = ...
        strread(tline, ['%d %s %f %f %f %f %f %d %f ' ...
             '%f %f %f %f %f %d %f %f %f %d %f'], 1, 'delimiter', ',');
    
    tline = fgets(fid);
    i = i+1;
end
nGen = i-1;

% 4. Read branch (actually line) data
tline = fgets(fid);
i = 1;
while strread(tline, '%d', 1) ~= 0
    [liFrom(i) liTo(i) liCKT liR(i) liX(i) liB(i) liRateA liRateB liRateC ...
            liGI liBI liGJ liBJ liStat(i) liLen liO liF] = ...
        strread(tline, ['%d %d %s %f %f %f %f %f %f' ...
             '%f %f %f %f %d %f %d %f'], 1, 'delimiter', ',');
    
    tline = fgets(fid);
    i = i+1;
end
nLine = i-1;

% 5. read transformer data

% Note that there are four lines for each. Only two-winding transformers.
tline = fgets(fid);
i = 1;
while strread(tline, '%d', 1) ~= 0
    [trFrom(i) trTo(i) trNull trCKT trCW trCZ trCM trMag1 trMag2 trNMetr ...
        trName trStat(i) trO trF] = ...
        strread(tline, '%d %d %d %s %d %d %d %f %f %d %s %d %d %f', 1, ...
                'delimiter', ',');
            
    tline = fgets(fid);
    [trR12(i) trX12(i) trSBase(i)] = strread(tline, '%f %f %f', 1, 'delimiter', ',');
    
    tline = fgets(fid); % can be ignored
    [trWindV1(i) trNomV1 trAngle(i) trRatA1 trRatB1 trRatC1 trCode trCont trRMA ...
        trRMI trVMA trVMI trNTP trTab trCR trCX] = ...
        strread(tline, '%f %f %f %f %f %f %d %f %f %f %f %f %d %d %f %f', 1, ...
                'delimiter', ',');

    tline = fgets(fid); % can be ignored
    [trWindV2(i) trNomV2] = strread(tline, '%f %f', 1, 'delimiter', ',');
    
    tline = fgets(fid);
    i = i+1;
end
nTransformer = i-1;

% 6. Skip area data
tline = fgets(fid);
while strread(tline, '%d', 1) ~= 0
    tline = fgets(fid);
end

% 7. Skip two terminal dc data
tline = fgets(fid);
while strread(tline, '%d', 1) ~= 0
    tline = fgets(fid);    
end

% 8. Skip switched shunt data
tline = fgets(fid);
i = 1;
while strread(tline, '%d', 1) ~= 0
    [swBus(i) swMode swVsLo swVsLo swRem swBInit(i) swN swB] = ...
        strread(tline, '%d %d %f %f %f %f %d %f', 1, 'delimiter', ',');
    
    tline = fgets(fid);
    i = i+1;
end
nSwitched = i-1;

% 9. Skip impedance correction data section
tline = fgets(fid);
while strread(tline, '%d', 1) ~= 0
    tline = fgets(fid);    
end

% 10. Skip multi-terminal data section
tline = fgets(fid);
while strread(tline, '%d', 1) ~= 0
    tline = fgets(fid);    
end

% 11. Skip multi-section line data section
tline = fgets(fid);
while strread(tline, '%d', 1) ~= 0
    tline = fgets(fid);    
end

% 12. read zone data section
tline = fgets(fid);
while strread(tline, '%d', 1) ~= 0
    tline = fgets(fid);    
end

% 13. Skip inner-area data section
tline = fgets(fid);
while strread(tline, '%d', 1) ~= 0
    tline = fgets(fid);    
end

% 14. read owner data section
tline = fgets(fid);
while strread(tline, '%d', 1) ~= 0
    tline = fgets(fid);
end

% 15. Read FACTS device data
tline = fgets(fid);
i = 1;
while strread(tline, '%d', 1) ~= 0
    [faNo faFrom(i) faTo(i) faMode faPd(i) faQd(i) faVset(i) faShCurrent ...
            faPtrmax faVmin faVmax faVsmax faImax faLinx faOwner] = ...
        strread(tline, '%d %d %d %d %f %f %f %f %f %f %f %f %f %f %d', 1, 'delimiter', ',');
    
    tline = fgets(fid);
    i = i+1;
end
nFACTS = i-1; 

% Finished reading all data blocks. As the data are in a silly format (at
% least for the purpose of this assigment, they should be merged. Bus,
% load, generator, switched shunt, and FACTS data can be put together.
% Also, line data and transformer data are from the same nature and are
% put in br___ vectors.

% Mix line and transformer data in br___ series vectors.
% Starting with lines, turns ratio is 1.
for i=1:nLine
    brFrom(i) = liFrom(i);
    brTo(i)   = liTo(i);
    brR(i)    = liR(i);
    brX(i)    = liX(i);
    brB(i)    = liB(i);
    brRatio(i)= 1;
    brStat(i) = liStat(i);    
end
% then add transformer, turns ratio is ratio/__ angle
for i=1:nTransformer
    brFrom(nLine+i) = trFrom(i);
    brTo(nLine+i)   = trTo(i);
    brR(nLine+i)    = trR12(i) * (baseMVA/trSBase(i));
    brX(nLine+i)    = trX12(i) * (baseMVA/trSBase(i));
    brB(nLine+i)    = 0 * (baseMVA/trSBase(i));
    brRatio(nLine+i)= trWindV1(i)/trWindV2(i) * exp(j*pi/180 * trAngle(i));
    brStat(nLine+i) = trStat(i);
end
nBranch = length(brFrom);

% Mix bus, load, generator, switched shunt, and FACTS data
buPd = zeros(nBus, 1);
buQd = buPd;
buPg = buPd;
buQg = buPd;
buQmax = 9999 * ones(nBus, 1);
buQmin = -buQmax;

% First add load data Pd and Qd
for i=1:nLoad
    buPd(loBus(i)) = loStat(i)*loPL(i);
    buQd(loBus(i)) = loStat(i)*loQL(i);
end

% then add generator data Pg and Qg and Q limits (actually no need to save Qg's)
for i=1:nGen
    buPg(geBus(i)) = geStat(i)*gePG(i);
    buQg(geBus(i)) = geStat(i)*geQG(i);
    buQmin(geBus(i)) = geStat(i)*geQmin(i);
    buQmax(geBus(i)) = geStat(i)*geQmax(i);
end

% and finally, add UPFC data
% The effect of a UPFC is modeled noticing that UPFC turns the sending bus
% into a PV bus as it sets the voltage magnitude for it and turns the
% recieving bus to a PQ one (it should be so already).

% Save buPd and buQd for future use
buOriginalPd = buPd;
buOriginalQd = buQd;
for i=1:nFACTS
    % set "From" bus type to PV
    buType(faFrom(i)) = PV_BUS;  
    % enforce voltage setpoint
    buVoltage(faFrom(i)) = faVset(i);
    % adjust power (Pd), (like normals loads)
    buPd(faFrom(i)) = buPd(faFrom(i)) + faPd(i);
    buQd(faFrom(i)) = buQd(faFrom(i)) + faQd(i);
    
    % set "To" bus type to PQ
    if buType(faTo(i)) ~= PQ_BUS
        disp(sprintf(['WARNING: Receiving bus of the UPFC (%d)'...
                ' should be already of type PQ.'],faTo(i)));
    end
    % adjust Pd and Pq (injected to the bus, as if they were negative loads)
    buPd(faTo(i)) = buPd(faTo(i)) - faPd(i);
    buQd(faTo(i)) = buQd(faTo(i)) - faQd(i);
end

% Close the file
fclose(fid);

% Display statistics of the system
disp(sprintf(['\nThe system has %d buses, %d loads, %d generators, '...
        '%d lines, %d transformers,'], nBus, nLoad, nGen, nLine,...
        nTransformer))
disp(sprintf('%d switched shunt capacitors, and %d UPFC''s.',...
        nSwitched, nFACTS))
    