% at_InitialStates.m
% This script calculates the initial value of the system states based on
% power flow solution (steady-state) values of system voltages and powers.
% As synchronous generators are modeled by the 6th order model, there are 6
% states per syn. gen. Five states are used for each exciter system. Also,
% each load bus has two states, x_p and x_q.
%
% Needs: 
%  P_G, Q_G, V_bus: Initial powers and voltages of the system
%             nGen: Number of generators
%
% Defines:
%                x: Vector of the system states; 6 for each SG, 5 for each
%                   exciter, 2 for each load bus
%
% Part of the code written for GPFM strategy,
% submitted to IEEE Transactions on Smart Grid, July 2011.
%
% Copyright (c) 2012 by Ali Mehrizi-Sani
% Washington State University
% mehrizi@eecs.wsu.edu
% http://eecs.wsu.edu/~mehrizi
% (509) 335-6249


% Find injected current from ground to the network at each bus
I_bus = (P_g - j*Q_g) ./ conj(V_bus);

% Just use I and V for generator buses
I_tgen = I_bus(genBus);
E_tgen = V_bus(genBus);

% Find mechanical torque; at ss, it is equal to electrical output power
T_m = P_g(genBus);

% Find delta angle, the angle between q-axis of the machine and R-axis of
% the common RI frame
delta = angle(E_tgen + (R_a + j*X_q).*I_tgen);

% Calculate dq voltages and currents 
ss = sin(delta); cc = cos(delta);
e_d = real(E_tgen).*ss - imag(E_tgen).*cc;
e_q = real(E_tgen).*cc + imag(E_tgen).*ss;
i_d = real(I_tgen).*ss - imag(I_tgen).*cc;
i_q = real(I_tgen).*cc + imag(I_tgen).*ss;

% Find field circuit current and voltage
i_fd = (e_q + R_a.*i_q + X_d.*i_d) ./ X_ad;
e_fd = R_fd.*i_fd;

% Caclulate initial states of the generators
dw     = zeros(nGen, 1);
delta  = delta;
psi_fd = (X_ad+X_fd).*i_fd - X_ad.*i_d;
psi_1d = X_ad .* (i_fd-i_d);
psi_1q = -X_aq.*i_q;
psi_2q = psi_1q;

% Steady-state exciter output current and voltages
% I_fd = X_adu * i_fd;
E_fd = (X_adu./R_fd) .* e_fd;

% PSS initial states
v_1 = abs(E_tgen);
v_2 = zeros(nGen, 1);
v_3 = zeros(nGen, 1);

% Exciter initial states
v_4 = E_fd ./ K_A;
V_ref = E_fd./K_A + v_1;

% Exponential recovery load model initial states
x_p = zeros(nBus, 1);
x_q = zeros(nBus, 1);

% Stator all states in a single vector
x = [dw
    delta
    psi_fd
    psi_1d
    psi_1q
    psi_2q
    v_1
    v_2
    v_3
    v_4
    E_fd
    x_p
    x_q
];