% at_Stability.m
% This script, Ali's Transient Stability, solves time-domain set of equations
% describing transient behavior of a power system. Dynamics of generators and
% their exciters, damper windings and power system stabilizers as well as loads
% are considered in this program. Calculation of initial conditions is based on
% the solution of power flow which is obtained by a_pf function previously
% written.
%
% The dynamic data (and power flow data) should be in PSS/E format. The
% program is strict about the format of file and there should be one
% exciter for each generator. Currently, 'GENROU', 'GENSAL', and 'SCRX'
% models are recognized. Also, the buses should be numbered in a way that
% load (pq) buses come first, followed by generator (slack and pv) buses.
%
% Needs:           NOTHING, uses default data files for the 12-bus
%                  test system
% Defines:         NOTHING
%
% Part of the code written for GPFM strategy,
% submitted to IEEE Transactions on Smart Grid, July 2011.
%
% Copyright (c) 2012 by Ali Mehrizi-Sani
% Washington State University
% mehrizi@eecs.wsu.edu
% http://eecs.wsu.edu/~mehrizi
% (509) 335-6249


% Begin with a fresh workspace
clear all

% Show welcome message
disp(sprintf('\nAli''s Transient Stability Simulatin Program.\nCopyright (c) 2006 Ali Mehrizi-Sani.'));

% Run power flow first, uses default 
disp(sprintf('\n\nRunning power flow solver...'));
a_pf
disp(sprintf('Finished running power flow solver.\n'));

% Clear not necessary power flow data and shape others
at_PowerFlowData;

% Read dyr file for dynamic data of generators and excitors and caclulate
% necessary reactances and resistances in system base
disp(sprintf('\nReading dynamic data file...'));
at_DyrRead;
disp(sprintf('DYR file read completed.'));

% Define share of each load that is to be modeled as Z, I, P, and ER and
% the parameters of ER model of P and Q components of each load
disp(sprintf('\nInitializing load types...'));
at_DefineLoadTypes;
disp(sprintf('Initialization of load types completed.'));

% Define Ybus for normal and faulted network
disp(sprintf('\nUpdating Ybus for linear loads...'));
at_DefineYbus;
disp(sprintf('Updating Ybus finished.'));

% Find initil values of states and store them in "x"
disp(sprintf('\nFinding initial values of states...'));
at_InitialStates;
disp(sprintf('Initial values calculated.'));


% Initialize the integrator
disp(sprintf(['\nInitializing the Gill''s version of '...
        'fourth-order Runge-Kutta method...']));
at_InitializeSolver;
disp(sprintf(['Finished initializing the integrator with a time-step of '...
        '%5.4f seconds\nand total simulation time of %3.2f seconds.'], dt, max(Time)));

% Define the time to apply fault and the time that fault is on. Set t_fault
% to a value higher than max(Time) to simulate steady-state conditions.
t_fault = 0.1;
t_clear = 0.1;
disp(sprintf(['\nRunning the integrator for a three-phase fault at bus 5 at t = %4.3f '...
        'seconds and\ncleared after %4.3f seconds...'], t_fault, t_clear));

% Initialize timer
tic

for curTime=Time
    iter = iter + 1;
    
    % Apply fault (or no fault) depending on the current time
    if curTime < t_fault
        % Pre-fault period
        Y_bus = Y_bus0;
        Z_gbus = Z_gbus0;
    elseif curTime < t_fault+t_clear
       % During fault period
        Y_bus = Y_busf;
        Z_gbus = Z_gbusf;        
    else
        % Post-fault period        
        Y_bus = Y_bus0;
        Z_gbus = Z_gbus0;        
    end

    % Gill's version of RK-4
    for jj=1:4
        % Caclulate time derivative of states and solve I(x, V) = Y_N * V
        % and store derivates in f
        xx = x(:, jj);
        at_DStates;
        f = px;

        % Gill's variables
        k(:, jj) = a(jj) * (f - b(jj)*q(:, jj));
        x(:, jj+1) = x(:, jj) + k(:, jj)*dt;
        q(:, jj+1) = q(:, jj) + 3*k(:, jj) - c(jj)*f;
	end
    
    % Update Gill's variables for use in the next time step
    q(:, 1) = q(:, 5);
    x(:, 1) = x(:, 5);
    
    % Store new states and current vus voltages
    Sol = [Sol x(:, 5)];
    allVoltages = [allVoltages V_bus];   
end
disp(sprintf('Finished running the integrator.'));
disp(sprintf('\nTime-domain simulation took %1.2f seconds to complete.', toc))

% Produce graphs
disp(sprintf('\nProducing output graphs...'));
at_OutputGraphs;
disp(sprintf('Output graphs completed.'));
disp(sprintf('\nRUN COMPLETED.'));